
#include "Global.H"
#include "SimStats.H"

using namespace std; 

SimStats::SimStats()
{
  num_simulations = 0; 
  num_players = 0;
  
  remaining_units = NULL;
  remaining_ttlhp = NULL; 
}

SimStats::~SimStats()
{
  DELETEARR(remaining_units);
  DELETEARR(remaining_ttlhp);
  DELETEARR(wins);
}

void SimStats::set_numplayers(int n)
{
  num_players = n; 
  
  remaining_units = new int[n];  
  FORS(i, n) remaining_units[i] = 0; 
  
  remaining_ttlhp = new int[n];
  FORS(i, n) remaining_ttlhp[i] = 0; 
  
  wins = new int[n]; 
  FORS(i, n) wins[i] = 0; 
}

void SimStats::start_timer()
{
  gettimeofday(&tv_start, NULL); 
}

void SimStats::stop_timer()
{
  gettimeofday(&tv_end, NULL);
}

void SimStats::add_stats(SimStats & stats)
{
  if (num_players != stats.num_players)
  {
    ERR("Adding two stats objects where num_players not equal"); 
  }
     
  num_simulations += stats.num_simulations; 
  
  FORS(i, num_players)
  {
    wins[i] += stats.wins[i]; 
    remaining_units[i] += stats.remaining_units[i];
    remaining_ttlhp[i] += stats.remaining_ttlhp[i];
  }
}

void SimStats::add_remaining_units(int player, int num)
{
  if (player >= num_players)
    ERR("SimStats::add_remaining_units player >= num_players"); 
  
  remaining_units[player] += num; 
}
 
void SimStats::add_remaining_ttlhp(int player, int num)
{
  if (player >= num_players)
    ERR("SimStats::add_remaining_units player >= num_players"); 
  
  remaining_ttlhp[player] += num; 
}

void SimStats::print()
{
  cout << "Number of simulations " << num_simulations << endl; 
  int delta_microsec =   (tv_end.tv_sec*1000000 + tv_end.tv_usec)
                       - (tv_start.tv_sec*1000000 + tv_start.tv_usec); 
  
  double delta_seconds = delta_microsec/(1000000.0); 
  
  cout << "Total time (seconds) = " << delta_seconds << endl;
  cout << "Avg time (seconds) per simulation = " << (delta_seconds / num_simulations) << endl ;  
    
  FORS(i, num_players)
  {
    double sims = (double)num_simulations; 
    double remunits = (double)remaining_units[i]/sims; 
    double remttlhp = (double)remaining_ttlhp[i]/sims; 
    
    cout << "Wins for player " << i << " = " << wins[i] << endl;
    cout << "Avg. num remaining units for player " << i << " = " << remunits << endl; 
    cout << "Avg. num remaining ttlhp for player " << i << " = " << remttlhp << endl; 
  }
}



