
#ifndef SIMULATION_H
#define SIMULATION_H

#include <vector>
#include <string>

#include "MiniGameState.H"
#include "Player.H"
#include "SimStats.H"

typedef MobileObj<MiniGameState> MobObj;

class Simulation
{
  std::vector<std::string> * trace; 
  char * parmsfile; 
  
  bool enable_gui; 
  int gui_delay;  
  int max_timesteps; 
  
protected:
  MiniGameState * statePtr;  
  MiniGameParameters * parmsPtr; 
  Player * player0Ptr; 
  Player * player1Ptr; 
  
  int timestep; 
  int winner;  
  bool players_fullview; 
  bool initial_wait; // wait to allow taking screenshots
  
  // Statistics collected over many simulations
  SimStats * statsPtr;  
             
  /* Override some of these when making a new simulation */  
  virtual void create_state();
  virtual void setup_players();    
  virtual void init_parms(); 
  virtual void init_state(); 
  
  virtual void pre_simloop(); 
  virtual void post_simloop();   
  virtual void pre_simiter(); 
  virtual void post_simiter(); 
  
  virtual void collect_endgame_stats(); 
  
public: 
  Simulation(); 
  virtual ~Simulation(); 
  
  void set_parmsfile(char * newfile) { parmsfile = newfile; }
  void set_enablegui(bool eg) { enable_gui = eg; }
  void set_guidelay(int gd) { gui_delay = gd; }
  void set_stats(SimStats * _sp) { statsPtr = _sp; }
  void set_maxtimesteps(int mts) { max_timesteps = mts; }
  void set_initial_wait(bool f) { initial_wait = f; }
  
  SimStats* get_stats() { return statsPtr; }
  
  void open_trace(char * filename);
  void play();
  
};

#endif
