
#include <string>
#include <iostream>
#include <fstream>
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/split.hpp>

#include "Features.H"
#include "MiniTacticalGameState.H"
#include "TacticalTrainingNNSim.H"

#include "floatfann.h"

using namespace std; 

TacticalTrainingNNSim::TacticalTrainingNNSim()
{
  enabledFtrs.push_back(FTR_HPDIFF);
  enabledFtrs.push_back(FTR_UNITCOSTDIFF);
}

TacticalTrainingNNSim::~TacticalTrainingNNSim()
{
}

void TacticalTrainingNNSim::create_state()
{
  statePtr = new MiniTacticalGameState;   
}

void TacticalTrainingNNSim::pre_simiter()
{
  if (timestep % 10 == 0) 
  {
    vector<double> ftrVals; 
    Features::snapshot(*statePtr, enabledFtrs, ftrVals);
    ostringstream oss; 
        
    FORALL (ftrVals, v) {
      oss << *v << " "; 
    }
        
    string ss = oss.str();
    string ssp = ss.substr(0, ss.length()-1);
    snapshots.push_back(ssp);
        
    //cout << "adding " << ssp << endl;        
              
    //cout << "Value of tactadv feature is " << fval << endl;
    //sumfval += fval; 
  }
}

void TacticalTrainingNNSim::post_simloop()
{
  bool calc_error = false; 
        
  if (winner != 2)
  {
    cout << "Appending neural net training data to data/rtstact.data and calc'ing error" << endl; 
    ofstream outfile("data/rtstact.data", ios_base::app); 
      
    struct fann *ann = NULL; 
    if (calc_error) ann = fann_create_from_file("data/rtstact.net");    
      
    FORALL (snapshots, ss)
    {
      cout << "."; 
      fflush(stdout); 
        
      outfile << (*ss) << endl;
      outfile << winner << endl;
        
      if (calc_error)
      {
        vector<string> parts;
        boost::split(parts, (*ss), boost::is_any_of(" ")); 
          
        fann_type *calc_out;
        fann_type input[2];
    
        input[0] = to_double(parts[0]);
        input[1] = to_double(parts[1]);
        calc_out = fann_run(ann, input);
    
          //printf("xor test (%f,%f) -> %f\n", input[0], input[1], calc_out[0]);
        double error = ABS(winner - calc_out[0]);
        cout << "inputs " << input[0] << " " << input[1] << " -> " << calc_out[0]
            << ", err = " << error << endl;     
      }
    }
      
    if (calc_error) fann_destroy(ann);
      
    outfile.close(); 
    cout << endl;
  }
}    

