// $Id: ol_main.C 12676 2012-10-11 21:49:35Z mburo $

// (c) Michael Buro
// licensed under GPLv3

#include <stdlib.h>

#include <fstream>
#include <string>
#include <vector>
#include <iostream>

#include "IMapSim.H"
#include "SimStats.H"
#include "Simulation.H"
#include "TacticalMissionSim.H"
#include "TacticalTrainingNNSim.H"
#include "TacticalExecNNSim.H"

using namespace std;

// Green is player 0
// Red is player 1

void usage()
{
  cout << "Usage: bin/ol [options]" << endl;
  cout << endl;
  cout << "Available options are: " << endl;
  cout << "   --nogui " << endl; 
  cout << "   --num <number of simulations>" << endl; 
  cout << "   --parms <filename>" << endl; 
  cout << "   --replay <filename>" << endl; 
  
  exit(-1);
}


void handle_clargs(int argc, char ** argv)
{
  //cout << "argc=" << argc << endl;
  //exit(-1);
  
}


int main(int argc, char ** argv)
{
  srandom(time(NULL));     
  
  char * tracefile = NULL;
  char * parmsfile = NULL; 
  int num_simulations = 1;
  bool enable_gui = true;  
  
  // handle command-line arguments
  for (int i = 1; i < argc; i++)
  {
    if (strcmp(argv[i], "--replay") == 0)
    {
      i++; if (i >= argc) usage();
      tracefile = argv[i];
    }
    else if (strcmp(argv[i], "--parms") == 0)
    {
      i++; if (i >= argc) usage();
      parmsfile = argv[i];
    }
    else if (strcmp(argv[i], "--num") == 0)
    {
      i++; if (i >= argc) usage();
      num_simulations = to_int(argv[i]); 
    }
    else if (strcmp(argv[i], "--nogui") == 0)
    {
      enable_gui = false; 
    }
    else
      usage();
  }
    
  SimStats stats;
  stats.set_numplayers(2);  
  stats.start_timer(); 
  
  // create and start the simulation    
  for (int i = 0; i < num_simulations; i++)
  {
    cout << endl;
    cout << "Running simulation " << (i+1) << endl; 
            
    // NOTE: If sim is changed, might also need to change # players in above!    
    Simulation sim; 
    //
    //TacticalTrainingNNSim sim;
    //TacticalExecNNSim sim;
    //IMapSim sim; 
    //TacticalMissionSim sim; 
    
    if (tracefile != NULL) 
      sim.open_trace(tracefile);
    
    sim.set_parmsfile(parmsfile); 
    
    sim.set_enablegui(enable_gui);
    sim.set_guidelay(1);
    
    sim.set_maxtimesteps(10000);

    sim.play();
    
    // collect the stats from the run
    SimStats* runstatsPtr = sim.get_stats(); 
    stats.add_stats(*runstatsPtr); 
  }
  
  stats.stop_timer(); 
  
  cout << " " << endl; 
  stats.print(); 

  return 0;
}

