
#include "math.h"

#include <vector>
#include <string>
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/split.hpp>

#include "Global.H"

using namespace std; 

void vector_test()
{
  // vector test
  vector<int> v;
  const int N = 1000;
  int count = 0;
    
  for (int i=0; i < N; i++) v.push_back(i);
  
  FORALL (v, i) {
    FORALL (v, j) {
      count++;
    }
  }
  
  cout << count << endl;
  exit(0);
}

void splitup_test()
{
  // NOTE: split is working properly in boost.. no need for this
  // anymore
   
  // splitup test
  string str = "hello,my,name,is";
  vector<string> vec; 
  splitup(vec, str, ","); 
    
  FORALL(vec, itr) 
      cout << "###" << (*itr) << "###" << endl;
    
  string str2 = "hello,^$my,^$name,^$is";
  vector<string> vec2; 
  splitup(vec2, str2, ",^$"); 
    
  FORALL(vec2, itr) 
      cout << "###" << (*itr) << "###" << endl;

  string str3 = "bla**bla**";
  vector<string> vec3; 
  splitup(vec3, str3, "**"); 
    
  FORALL(vec3, itr) 
      cout << "###" << (*itr) << "###" << endl; 
    
  exit(0);
}

void join_test()
{
  // join test
  vector<string> vec;
  vec.push_back("hello");
  vec.push_back("my");
  vec.push_back("name");
  vec.push_back("is");
    
  cout << join(vec, ",", 2) << endl;
  exit(0);
}

void trim_test()
{
    // trim test (NOTE: There is one in boost!)
  string str = "   \t \r qklsd dfj  \t asl;k   \n   ";
  string newstr = trim(str);
  cout << "##" << newstr << "##" << endl; 
  exit(0);
}
    
void split_test()
{
    // split test
  string t = " # 1,2,3    4,5,6";
  
  Vector<std::string> actions;
  
  split(actions, t, boost::is_any_of(" "), boost::token_compress_on); 
  
  FORALL (actions, i) {
    cout << '<' << *i << '>' << endl;
  }
  
  exit(0);
}

void line_circle_intersection_test()
{
  // eqs take form of y = mx+b and (x-k1)^2 + (y-k2)^2 = r^2
  
  double m, b; 
  double k1, k2, r; 
  double A, B, C;
  double x1, x2; 
  double y1, y2; 
    
  // simple test, y = x and (x-5)^2 + (y-5)^2 = 9  
  m = 1; b = 0; k1 = 5; k2 = 5; r = 3; 
  
  // see analysis on paper
  A = (1+m*m); 
  B = (-2*k1 + 2*b*m - 2*k2*m);
  C = (k1*k1 + b*b - 2*b*k2 + k2*k2 - r*r); 
  
  // quadr eq. 
  x1 = (-B - sqrt(B*B - 4*A*C))/(2*A); 
  x2 = (-B + sqrt(B*B - 4*A*C))/(2*A); 
  
  y1 = m*x1 + b; 
  y2 = m*x2 + b; 
  
  cout << x1 << "," << y1 << "   " << x2 << "," << y2 << endl; 
  
  // simple test, y = 2x-3 and (x-3)^2 + (y+2)^2 = 9  
  m = 2; b = -3; k1 = 3; k2 = -2; r = 3; 
  
  // see analysis on paper
  A = (1+m*m); 
  B = (-2*k1 + 2*b*m - 2*k2*m);
  C = (k1*k1 + b*b - 2*b*k2 + k2*k2 - r*r); 
  
  // quadr eq. 
  x1 = (-B - sqrt(B*B - 4*A*C))/(2*A); 
  x2 = (-B + sqrt(B*B - 4*A*C))/(2*A); 
  
  y1 = m*x1 + b; 
  y2 = m*x2 + b; 
  
  cout << x1 << "," << y1 << "   " << x2 << "," << y2 << endl; 
  
  
}

int main()
{
  // call testing function here
  line_circle_intersection_test(); 
}
